var STATUS_NORMAL   = 0;  // 通常
var STATUS_LOADING  = 1;  // 読み込み中

function CDataMgr(option) {
	var oDate = new Date();
	this._id = oDate.getTime();

	this._status = STATUS_NORMAL;

	/**
	 * auto     trueにするとデータ操作の度サーバーデータも同じように更新する
	 * !param   データ定義、フィールド名と型名の連想配列
	 * recv_dat
	 *     url     データ受信URL
	 *     success 受信完了時に呼び出される関数
	 *     failure 受信失敗時に呼び出される関数
	 * send_dat
	 *     url     データ送信用のURL
	 * recv_csv
	 *     url     CSVデータダウンロードのURL
	 * send_csv
	 *     url     CSVデータアップロードのURL
	 * show
	 *     base      表示DOM作成の元となるDOMのセレクタ
	 *     base_hide 表示後、元DOMを隠すかどうか(true=隠す,false=隠さない)
	 *     replace   置き換え元キー名と置き換え先タグの連想配列、指定が無い場合はデータのキー名をそのまま使用
	 * add
	 *     selector  フィールド名とフォームIDの連想配列
	 *     def_param   フィールド名とフォームIDの連想配列
	 * set
	 *     selector  フィールド名とフォームIDの連想配列
	 */
	this._option = option;

	/**
	 * 受信したデータ
	 * @var array
	 */
	this._data = [];

	/**
	 * 差分データ
	 * 受信したデータを更新を差分として保持。更新データとしてサーバーに送る時に使用。
	 * @var array
	 */
	this._data_diff = [];



	// デフォルト値設定
	{
		var key;
		if ('undefined' == typeof(this._option.uniq_id)) {
			this._option.uniq_id = 'id';
		}
		if ('undefined' == typeof(this._option.show.replace)) {
			this._option.show.replace = {}
		}
		if ('undefined' == typeof(this._option.show.base_hide)) {
			this._option.show.base_hide = false;
		}
		if ('undefined' == typeof(this._option.add)) {
			this._option.add = {}
		}
		if ('undefined' == typeof(this._option.set)) {
			this._option.set = {}
		}
		// 追加時の初期値
		if ('undefined' == typeof(this._option.add.def_param)) {
			this._option.add.def_param = {}
		}
		for (key in this._option.param) {
			if ('undefined' == typeof(this._option.add.def_param[key])) {
				this._option.add.def_param[key] = '';
			}
		}
		// 必ず送信する項目の初期化
		if ('undefined' == typeof(this._option.set.send)) {
			this._option.set.send = {}
		}
	}

	/**
	 * 表示
	 */
	this.show = function() {
		var i;
		var obj = jQuery(this._option.show.base);
		var html, html_base = obj.html();
		var key, obj_form, form_name;
		var data = this.getAllData();

		obj.show();

		jQuery('.CDataMgr_record_' + this._id).remove();

		obj_prev = obj;
		for (i = 0; i < data.length; i ++) {
			if (null == data[i]) {
				continue;
			}
			html = this._template(html_base, this._option.show.replace, data[i]);
			obj_prev = jQuery(obj)
				.clone()
				.addClass('CDataMgr_record_' + this._id)
				.html(html)
				.insertAfter(obj_prev)
			;
			// 対応するフォームがあったら設定
			for (key in this._option.param) {
				if ('undefined' == typeof(data[i][key])) {
					continue;
				}

				form_name = key + '_' + i;
				// ID から検索
				obj_form = obj_prev.find('#' + form_name);
				// 名前から検索
				if (0 == obj_form.length) {
					obj_form = obj_prev.find('[name="' + form_name + '"]');
				}
				// 名前でも見つからなかったら次
				if (0 == obj_form.length) {
					continue;
				}

				switch (obj_form.attr('type')) {
				case 'radio':
				case 'checkbox':
				case 'select':
				case 'select-multiple':
					data[i][key] = data[i][key].split(',');
				default:
					obj_form.val(data[i][key]);
					break;
				}
			}
		}

		if (true == this._option.show.base_hide) {
			obj.hide();
		}

		if ('undefined' != typeof(this._option.show.callback)) {
			this._option.show.callback();
		}
	}

	//------------------------------------------------------------------------------------------------------------------
	/**
	 * サーバーからデータを受信する
	 */
	this.recv_dat = function() {
		oDataMgr = this;

		this._serverCall(
			this._option.recv_dat,
			null,
			function(data) {
				// 内部に保存
				oDataMgr._data = data.data;

				// 差分初期化
				oDataMgr.clearDiff();

				// コールバック呼び出し
				if ('undefined' != typeof(oDataMgr._option.recv_dat.success)) {
					oDataMgr._option.recv_dat.success(oDataMgr._data);
				}
			},
			function(data) {
			}
		);
	}

	/**
	 * サーバーへデータを送信する
	 */
	this.send_dat = function() {
		var i, j, key, val;

		this.form_update();

		// 必ず送信する項目
		if ('*' == this._option.set.send) {
			this._data_diff = this.getAllData(false);
		} else {
			for (i = 0; i < this._data_diff.length; i ++) {
				for (j = 0; j < this._option.set.send.length; j ++) {
					key = this._option.set.send[j];
					if ('undefined' == typeof(this._data_diff[i][key])) {
						this._data_diff[i][key] = this._data[i][key];
					}
				}
			}
		}

		// 開始コールバック呼び出し
		if ('undefined' != typeof(this._option.send_dat.start)) {
			var data = this._option.send_dat.start(this._data_diff);
			if (null == data) {
				return;
			}
			this._data_diff = data;
		}

		// 削除対象は uniq_id だけ渡す
		for (i = 0; i < this._data_diff.length; i ++) {
			if (null == this._data_diff[i] && 'undefined' != typeof(this._data[i])) {
				this._data_diff[i] = {}
				this._data_diff[i][this._option.uniq_id] = this._data[i][this._option.uniq_id];
			}
		}

		var oDataMgr = this;

		this._serverCall(
			this._option.send_dat,
			this._data_diff,
			function(data) {
				// 差分を更新
				oDataMgr._data = oDataMgr.getAllData();
				var i;
				for (i = oDataMgr._data.length - 1; i >= 0; i --) {
					if (null == oDataMgr._data[i]) {
						oDataMgr._data.splice(i, 1);
					}
				}

				// 差分初期化
				oDataMgr.clearDiff();

				// コールバック呼び出し
				if ('undefined' != typeof(oDataMgr._option.send_dat.success)) {
					oDataMgr._option.send_dat.success(oDataMgr._data);
				}
			},
			function(data) {
			}
		);
	}
	//------------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------------
	/**
	 * サーバーのデータをCSVでダウンロードする
	 */
	this.recv_csv = function() {
	}

	/**
	 * サーバーへデータをCSVでアップロードする
	 */
	this.send_csv = function() {
	}
	//------------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------------
	/**
	 * データ数を取得する
	 * @param boolean bDeleteItem 削除データも数えるかどうか
	 */
	this.get_count = function(bDelete) {
		var data = this.getAllData();
		var count = data.length;

		// 削除データは数えない
		if (true != bDelete) {
			for (i = 0; i < data.length; i ++) {
				if (null == data[i]) {
					count --;
				}
				alert(this._data_diff[i]);
				for (var key in data[i]) {
					alert(key + '[' + data[i][key] + ']');
				}
			}
		}

		return count;
	}

	/**
	 * データを取得する
	 */
	this.get = function(index, key) {
		var data = this.getAllData();
		if (null != data[index] && 'undefined' != typeof(data[index][key])) {
			return data[index][key];
		} else {
			return '';
		}
	}

	/**
	 * データを入れ替える
	 */
	this.swap = function(index_src, index_dst, key) {
		var key_list, tmp;

		this.form_update();
		var data = this.getAllData();

		// データが存在するか
		if ('undefined' == typeof(this._data_diff[index_src]) || 'undefined' == typeof(this._data_diff[index_dst])) {
			return this;
		}

		// 対象キーのリストを作成
		if ('undefined' != typeof(key)) {
			key_list = {};
			key_list[key] = '';
		} else {
			key_list = data[index_src];
		}

		// 入れ替え
		for (key in key_list) {
			tmp = data[index_src][key];
			this._data_diff[index_src][key] = data[index_dst][key];
			this._data_diff[index_dst][key] = tmp;
		}

		// サーバー更新
		if (true == this._option.auto) {
			this.send_dat();
		}

		return this;
	}

	/**
	 * データを複製する
	 */
	this.copy = function(index) {
		var data = this.getAllData();
		var i, key, obj, uniq_format;
		var copy_index;
		var new_value;

		var uniq_list = {};
		if ('undefined' != typeof(this._option.copy.uniq)) {
			uniq_list = this._option.copy.uniq;
		}

		obj = {}
		for (key in data[index]) {
			// 一意になるように値を設定する
			if ('undefined' != typeof(uniq_list[key])) {
				copy_index = 2;

				// 新しい値
				new_value = uniq_list[key];
				new_value = new_value.split('[value]').join(data[index][key]);
				new_value = new_value.split('[index]').join(copy_index);

				for (i = 0; i < data.length; i ++) {
					// 同じ値はあるか？
					if (new_value == data[i][key]) {
						copy_index ++;
						// 新しい値
						new_value = uniq_list[key];
						new_value = new_value.split('[value]').join(data[index][key]);
						new_value = new_value.split('[index]').join(copy_index);
						i = 0;
					}
				}

				// 複製
				obj[key] = new_value;
			} else {
				// 複製
				obj[key] = data[index][key];
			}
		}

		this._data_diff.push(obj);

		// サーバー更新
		if (true == this._option.auto) {
			this.send_dat();
		}

		return this;
	}

	/**
	 * データを編集する
	 * @param integer index 編集するデータのインデックス番号
	 */
	this.edit = function(index) {
		var data = this.getAllData();
		var key;

		for (key in this._option.set.selector) {
			jQuery(this._option.set.selector[key]).val(data[index][key]);
			jQuery(this._option.set.selector[key]).data('nj_index', index);
		}

		return this;
	}

	/**
	 * データを作成する
	 */
	this.add = function() {
		var oParam = {};
		var key;

		this.form_update();

		for (key in this._option.param) {
			oParam[key] = this._option.add.def_param[key];
		}

		this._data_diff.push(oParam);

		if (true == this._option.auto) {
			this.send_dat();
		}

		return this;
	}

	/**
	 * データを更新する
	 */
	this.set = function(index, key, val) {
		this.form_update();

		if ('undefined' != typeof(index) && 'undefined' != typeof(key) && 'undefined' != typeof(val)) {
			// 単一データ
			if (null != this._data_diff[index]) {
				this._data_diff[index][key] = val;
			}
		} else {
			if ('undefined' != typeof(this._option.set.selector)) {
				// 全データ
				for (key in this._option.param) {
					if (1 == jQuery(this._option.set.selector[key]).length) {
						index = jQuery(this._option.set.selector[key]).data('nj_index');
						this._data_diff[index][key] = jQuery(this._option.set.selector[key]).val();
						jQuery(this._option.set.selector[key]).val('');
					}
				}
			}
		}

		if (true == this._option.auto) {
			this.send_dat();
		}

		return this;
	}

	/**
	 * データを削除する
	 * @param integer index    削除するデータのインデックス番号
	 * @param boolean bNoCheck 確認せずに処理するかどうか
	 */
	this.del = function(index, bNoCheck) {
		this.form_update();

		if (true != bNoCheck && 'undefined' != typeof(this._option.del.confirm)) {
			if (!confirm(this._option.del.confirm)) {
				return this;
			}
		}

		this._data_diff[index] = null;

		if (true == this._option.auto) {
			this.send_dat();
		}

		return this;
	}

	/**
	 * 差分初期化
	 */
	this.clearDiff = function() {
		this._data_diff = [];
		var i;
		for (i = 0; i < this._data.length; i ++) {
			this._data_diff[i] = {}
		}

		return this;
	}
	//------------------------------------------------------------------------------------------------------------------

	/**
	 * サーバーデータと差分を組み合わせた最新のデータを取得する
	 */
	this.getAllData = function(bUseIndex) {
		var data;
		var i, key;

		if ('undefined' == typeof(bUseIndex)) {
			bUseIndex = true;
		}

		data = [];
		for (i = 0; i < this._data.length; i ++) {
			data[i] = {}
			for (key in this._data[i]) {
				data[i][key] = this._data[i][key];
			}
		}

		for (i = 0; i < this._data_diff.length; i ++) {
			if (null != this._data_diff[i]) {
				for (key in this._data_diff[i]) {
					if ('undefined' == typeof(data[i])) {
						data[i] = {}
					}
					data[i][key] = this._data_diff[i][key];
				}
				if (true == bUseIndex) {
					data[i]['nj_index'] = i;
				}
			} else {
				data[i] = null;
			}
		}

		return data;
	}

	/**
	 * サーバーに要求を送る
	 * @param object   option    サーバー要求のオプション
	 * @param object   param     サーバーに送るデータ
	 * @param function fnSuccess 通信成功時に呼び出すコールバック
	 * @param function fnFailure 通信失敗時に呼び出すコールバック
	 */
	this._serverCall = function(option, param, fnSuccess, fnFailure) {
		var ajax_option = {
			async:false,
			url:option.url,
			success:function(data, status) {
				eval('var oParam = (' + data + ');');

				if (0 <= oParam.code) {
					if (fnSuccess) {
						fnSuccess(oParam);
					}
				} else {
					if (fnFailure) {
						fnFailure(oParam);
					}
				}
			},
			failure:function(data, status) {
				var oParam = {
					code:-99999,
					error:'server connect error'
				};
				if (fnFailure) {
					fnFailure(oParam);
				}
			}
		}
		if (null != param) {
			ajax_option.type = 'POST';
			ajax_option.data = 'data=' + JSON.stringify(param);
		}

		jQuery.ajax(ajax_option);
	}

	/**
	 * 対応フォームと同期
	 */
	this.form_update = function() {
		var i;
		var obj, obj_form;
		var form_name;
		var data = this.getAllData();

		for (i = 0; i < data.length; i ++) {
			obj = jQuery('.CDataMgr_record_' + this._id).get(i);
			obj = jQuery(obj);
			for (key in this._option.param) {
				if (null == data[i] || 'undefined' == typeof(data[i][key])) {
					continue;
				}

				form_name = key + '_' + i;
				// ID から検索
				obj_form = obj.find('#' + form_name);
				if (0 != obj_form.length) {
					val = obj_form.val();
				}
				// 名前から検索
				else {
					obj_form = obj.find('[name="' + form_name + '"]');
					// 名前でも見つからなかったら次
					if (0 == obj_form.length) {
						continue;
					}
					// 
					val = [];
					obj_form.each(function() {
						if (this.checked) {
							val.push(jQuery(this).val());
						}
					});
					val = val.join(',');
				}

				// 元データから変わっているか？
				if (data[i][key] != val) {
					// 差分に登録
					this._data_diff[i][key] = val;
				}
			}
		}
	}

	/**
	 * njScriptテンプレート展開
	 * @param string sHtmlBase njScriptテンプレート
	 * @param object oReplace  置き換え元キー名と置き換え先タグの連想配列、指定が無い場合はデータのキー名をそのまま使用
	 * @param object oParam    置き換えに使うパラメータ
	 * @param string sTagFirst njScriptテンプレートのタグ開始文字
	 * @param string sTagLast  njScriptテンプレートのタグ終了文字
	 */
	this._template = function(sHtmlBase, oReplace, oParam, sTagFirst, sTagLast) {
		var sHtml = sHtmlBase;
		var key, key_replace;

		if ('undefined' == typeof(sTagFirst)) {
			sTagFirst = '[';
		}
		if ('undefined' == typeof(sTagLast)) {
			sTagLast = ']';
		}

		for (key in oParam) {
			if ('undefined' != typeof(oReplace[key])) {
				key_replace = oReplace[key];
			} else {
				key_replace = key;
			}
			key_replace = sTagFirst + key_replace + sTagLast;
			sHtml = sHtml.split(key_replace).join(oParam[key]);
		}

		return sHtml;
	}
}
