/**
 * njテーブル生成
 * @param object obj_top 作成対象のjQueryオブジェクト
 * @param object args    作成時に渡された引数
 */
function nj_create_table(obj_top, args) {
	if ('undefined' == typeof(args.page))  { args.page  =  1; }
	if ('undefined' == typeof(args.count)) { args.count = 10; }
	if ('undefined' == typeof(args.range)) { args.range = 10; }

	var option = {
		args : args,

		obj_load       : obj_top.find('.nj-table-load'),
		obj_none       : obj_top.find('.nj-table-none'),
		obj_root       : obj_top.find('.nj-table-root'),
		obj_head       : obj_top.find('.nj-table-head'),
		obj_base       : obj_top.find('.nj-table-base'),
		obj_page       : obj_top.find('.nj-table-page'),
		obj_page_base  : obj_top.find('.nj-table-page-base'),

		obj_page_prev  : obj_top.find('.nj-table-page-prev'),
		obj_page_this  : obj_top.find('.nj-table-page-this'),
		obj_page_other : obj_top.find('.nj-table-page-other'),
		obj_page_next  : obj_top.find('.nj-table-page-next'),

		record_list : [],
		url         : '',
		filter      : null,

		/**
		 * レコードを読み込む
		 * @param object option オプション情報を格納した連想配列
		 */
		load : function (option) {
			if (true == option.no_cache) {
				this.clear(option.url);
			}

			this.url    = option.url;
			if ('undefined' != option.filter) {
				this.filter = option.filter;
			}

			var obj_table = this;

			this.hide();
			this.obj_load.show();

			nj.record.get(this.url, function(record) {
				var max_count = eval('record.model.count');

				obj_table.record_list = eval('record.' + obj_table.args.list);
				if ('undefined' != typeof(max_count)) {
					var list  = obj_table.record_list;
					var index = (obj_table.args.page - 1) * obj_table.args.count;
					var last  = index + obj_table.args.count;

					var i;
					var j = 0;
					obj_table.record_list = [];
					for (i = 0; i < max_count; i ++) {
						if (index <= i && i < last) {
							obj_table.record_list[i] = list[j];
							j ++;
						} else {
							obj_table.record_list[i] = {};
						}
					}
				}
			});

			return this;
		},

 		/**
		 * キャッシュをクリアする
		 */
		clear : function() {
			nj.record.clear(this.url);
		},

		/**
		 * 次のページへ移動する
		 */
		next : function () {
			this.args.page ++;
			return this;
		},

		/**
		 * 前のページへ移動する
		 */
		prev : function () {
			this.args.page --;
			return this;
		},

		/**
		 * レコードを登録する
		 * @param string   url      登録処理のURL
		 * @param object   param    登録処理に渡すパラメータ
		 * @param function callback 登録処理後に呼び出す関数
		 */
		set : function (url, param, callback) {
			nj.record.request(url, param, callback);
			return this;
		},

		/**
		 * レコードを削除する
		 * @param string   url      削除
		 * @param function callback 削除処理後に呼び出す関数
		 */
		del : function (url, callback) {
			nj.record.request(url, {}, callback);
			return this;
		},

		/**
		 * レコード内のデータを取得する
		 * @param number index 取得するレコードの番号
		 * @param string key   取得する列の名前
		 */
		get : function (index, key) {
			if ('undefined' == typeof(key)) {
				return this.record_list[index];
			}

			return this.record_list[index][key];
		},

		/**
		 * レコード数を取得する
		 */
		getCount : function () {
			return this.record_list.length;
		},

		/**
		 * レコードを全て取得する
		 * @param object filter フィルタリング情報を格納した連想配列
		 */
		getAll : function (filter) {
			var key_filter;
			var bShow;
			var show_list = [];
			if ('undefined' != typeof(filter)) {
				// フィルターを通す
				for (i = 0; i < this.record_list.length; i ++) {
					bShow = true;
					for (key_filter in filter) {
						if (this.record_list[i][key_filter] != filter[key_filter]) {
							bShow = false;
							break;
						}
 					}
					if (true == bShow) {
						show_list.push(this.record_list[i]);
					}
 				}
			} else {
				// フィルター指定が無いので全て利用
				show_list = this.record_list;
			}

			return show_list;
		},

		/**
		 * 表示する
		 * @param object option 表示の細かい指定
		 */
		show : function (option) {
			if (null == option) {
				option = {}
			}

			this.hide();

			show_list = this.getAll(this.filter);

			// 表示位置補正
			if (this.args.page > show_list.length) {
				this.args.page = show_list.length;
			}
			if (this.args.page < 1) {
				this.args.page = 1;
			}

			// レコードを全て削除
			this.obj_root.find('.nj-table-record').remove();

			// テーブル構築
			var i, key, base, html;
			var page_max;
			var loop_start, loop_end, count;

			// 最大ページ数
			page_max = Math.ceil(show_list.length / this.args.count);

			// ベースHTML誌錡
			base = this.obj_base.html();

			// 全レコード処理
			loop_start = (this.args.page - 1) * this.args.count;
			count = 0;
			for (i = loop_start; i < show_list.length; i ++) {
				html = base;
				// データ置き換え
				for (key in show_list[i]) {
					html = html.split('[' + key + ']').join(show_list[i][key]);
				}
				// レコードの位置を渡す
				html = html.split('[record-index]').join(count);
				// レコード追加
				this.obj_base
					.clone()
					.removeClass('nj-table-base')
					.addClass('nj-table-record')
					.insertBefore(this.obj_base)
					.html(html)
					.show()
				;

				// 表示数増加
				count ++;
				// 表示数に達したか？
				if (this.args.count <= count) {
					// レコード追加完了
					break;
				}
			}

			// ページャ設定
			html = '';
			var html_tpl;
			var page_count;
			{
				// 前へ
				if (1 != this.args.page) {
					html += this.obj_page_prev.html();
				}
				// ページ
				if (this.obj_page_this.html() && this.obj_page_other.html()) {
					loop_end   = Math.min(this.args.page + this.args.range - 1, page_max);
					loop_start = Math.max(loop_end - this.args.range - this.args.range + 1, 1);
					page_count = 0;
					for (i = 1; i <= page_max && i <= loop_end; i ++) {
						if (loop_start <= i) {
							// ページ内容誌錡
							if (i == this.args.page) {
								html_tpl = this.obj_page_this.html();
							} else {
								html_tpl = this.obj_page_other.html();
							}
							// njScriptテンプレート置き換え
							html_tpl = html_tpl.split('[page]').join(i);
							// HTML追記
							html += html_tpl;
							// 
							page_count ++;
						}
					}
				}
				// 次へ
				if (page_max > this.args.page) {
					html += this.obj_page_next.html();
				}
				// HTML設定
				this.obj_page.html(html);
			}

			// テーブルソート機能設定
			this.obj_root.tablesorter();

			if (0 != count) {
				this.obj_root.show();
				this.obj_head.show();
			} else {
				this.obj_none.show();
			}

			this.obj_page.show();

			return this;
		},

		/**
		 * 隠す
		 */
		hide : function () {
			this.obj_load.hide();
			this.obj_none.hide();
			this.obj_root.hide();
			this.obj_head.hide();
			this.obj_base.hide();
			this.obj_page.hide();
			this.obj_page_base.hide();

			return this;
		}
	}

	nj.create_object(obj_top, option);
}
